/**
* \file ds75lv.c
* @brief Funktionsumfang zum Temperatursensor DS75LV
* \author Lars Neubert
* \date Feb 2010
*/

#include "twi.h"
#include "ds75lv.h"



/**
* Wandelt bergebene Vor- und Nachkommastellen durch Festkommaarithmetik in eine Zeichenkette um.
* @param vork Angabe der Vorkommastelle
* @param nachk Angabe der Nachkommastelle
* @param tstring Pointer der Zeichenkette, in welche Ergebnis gespeichert werden soll (muss mind 10Bytes beinhalten, z.B. '-125.1250'+'Stringende')
*/
void ds75lv_temptoa(int8_t vork, int8_t nachk, char* tstring)
{
	uint8_t i; 
	int16_t tmp;

	tstring[9]='\0';			//Stringende-Zeichen
	tmp=nachk>>4;			//laut Datenblatt nur ersten 4 Bit fr Nachkomma
	if( vork < 0 ) {		//Vorzeichen hinzufgen
    	tstring[0] = '-';              
    	vork = -vork;
		//1 auffllen geaschieht automatisch wenn msb=1
		//tmp|=0xFFF0;		//wenn negativ, dann durch >>4 entstandenen 0 mit 1 ersetzen, damit noch negative Zahl
		tmp=-tmp; 
	}
	else {
		tstring[0] = ' ';
		tmp&=~(0xFFF0);		//falls pos Temp, aber nachk=0b10000000 --> durch msb=1 und >>4 wird nach vorne automatisch mit 1 aufgefllt, da C denkt dass neg. Zahl
	}
 
	for(i=3; i>=1; i--) {
		tstring[i]=(vork % 10) +'0';	//Schleife rckwrts wegen modulo, '0' Addieren um ASCII-Wert fr Zahl zu erhalten
		vork /= 10;						//nchste Stelle (kein Komma, da variable 'vork' kein float)
  	}

	tstring[4]=',';

	tmp=tmp*625;						//eigentlich *0,0625, aber *625 um float zu vermeiden
	for(i=4; i>=1; i--) {
		tstring[4+i]=(tmp % 10) +'0';	//Schleife rckwrts wegen modulo, '0' Addieren um ASCII-Wert fr Zahl zu erhalten
		tmp /= 10;						//nchste Stelle (kein Komma, da vork kein float)
  	}
}

/**
* Initialisiert die TWI Schnittstelle und den Temperatursensor.
* Die Ausgabe wird auf 12Bit Auflsung gestellt.
*/
void ds75lv_init()
{	int8_t error=0;

	twi_init();
	error=twi_start(0b1001000,TWI_WRITE);
	if (error!=true) {error=twi_write(0b00000001);}		//Pointer Byte --> Configuration
	if (error!=true) {error=twi_write(0b01100000);}		//Data Byte --> 12Bit Genauigkeit
	twi_stop();

	if (error!=true) {error=twi_start(0b1001000,TWI_WRITE);}
	if (error!=true) {error=twi_write(0b00000000);}		//Pointer Byte --> Temperatur
	twi_stop();
}

/**
* Liet die Temperatur des Sensors DS75LV aus.
* @param vor Pointer auf Speicherstelle fr Vorkommazahl
* @param nach Pointer auf Speicherstelle fr Nachkommazahl
* @return gibt an, ob ein Timeout ausgelst wurde (=1 Timeout und somit Fehler, =0 alles i.O.)
*/
uint8_t ds75lv_get_temp(uint8_t* vor, uint8_t* nach)
{	int8_t error=0;

	#if shoutdown==1
	error=twi_start(0b1001000,TWIwrite);
	if (error!=true) {error=twi_write(0b00000001);}		//Pointer Byte --> Configuration
	if (error!=true) {error=twi_write(0b01100000);}		//Data Byte --> Normal Mode einstellen (aus ShutDownMode "aufwecken")
	if (error!=true) {error=twi_start(0b1001000,TWI_WRITE);}
	if (error!=true) {error=twi_write(0b00000000);}		//Pointer Byte --> Temperatur
	#endif

	error=twi_start(0b1001000,TWI_READ);
	if (error!=true) {
		error=twi_read_ack(vor);
		if (error!=true) {error=twi_read_nack(nach);}	
	}
	//ShoutDownMode
	#if shoutdown==1
	if (error!=true) {error=twi_start(0b1001000,TWI_WRITE);}
	if (error!=true) {error=twi_write(0b00000001);}		//Pointer Byte --> Configuration
	if (error!=true) {error=twi_write(0b01100001);}		//Data Byte --> ShutDownMode einstellen
	#endif		

	twi_stop();
	return error;
}



