/** \mainpage Siemens Handy
* \date Feb 2010
* \author Lars Neubert, neubert.lars@googlemail.com
* 
* \brief Projekt zur Kommunikation mit Mobiltelefonen
*
* Das Projekt dient zur Kommunikation mit Mobiltelefonen.
*<br> Das beinhaltet:
* - die Steuerung des Mobiltelefons mittels AT-Kommandos
* - die Mglichkeit zum Versenden von Textnachrichten im PDU Format
* - die Mglichkeit des Empfangs von Textnachrichten im PDU Format und anschlieender Dekodierung
* - das Auslesen der Datums- und Zeitangaben des angeschlossenen Mobiltelefons
*
* Die eigentliche Kommunikation mit dem Mobiltelefon erfolgt dabei ber die serielle UART Schnittstelle 
*<br> des Mikrocontrollers. Zur Vereinfachung wurde dabei die Standardein und -ausgabe auf die blichen
*<br> C Funktionen wie printf, gets, scanf, etc. umgeleitet. 
*<br> Neben der Kommunikation mit dem angeschlossenen Mobiltelefon, besteht zudem die Mglichkeit einen
*<br> digitalen I2C Sensor anzusteuern. Beispielhaft ist durch die Verwendung des TWI Interfaces des 
*<br> Mikrocontrollers eine Buskommunikation mit dem Temperatursensor DS75LV (und quivalente) in diesem  
*<br> Projekt implementiert.
*/

/**
* \file main.c
* @brief Hauptdatei des Projektes
* \author Lars Neubert
* \date Feb 2010
*/

#include "main.h"

/**
* globales Flag, welches Signalisiert, dass Timer1 ausgelst wurde
*/
volatile uint8_t timer_flag;


int main(void) {

	uint8_t vor=0;
	uint8_t nach=0;
	uint8_t error=0;
	uint8_t state=0;
	char s[20]={'0'};

	unsigned char year=0;
	unsigned char month=0;
	unsigned char day=0;
	unsigned char hour=0;
	unsigned char minute=0;
	unsigned char second=0; 

	DDRD |= (1<<PD5) | ( 1 << PD6 );

	lcd_init();	
	ds75lv_init();

    // 16 bit timer:  
    TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);			// CTC + Prescaler 1024 --> CPU/1024
	//TCCR1B = (1<<WGM12)|(1<<CS11)|(1<<CS10);			// CTC + Prescaler 256 --> CPU/256
	// bei 16MHz und prescaler 1024 --> 1s --> OCR1A  = 15624
	// bei 1MHz und prescaler 64 --> 1s	--> OCR1A  = 15624
    OCR1A  = 15624;							//vorladen, f_comp=fosc/(prescaler*(OCRnX+1)); OCRnX=fosc/(f_comp*prescaler)-1
    TIMSK |= (1<<OCIE1A);        			//Output Compare Match A aktivieren

    // 8 bit timer:  
    TCCR2 = (1<<WGM21)|(1<<CS22)|(1<<CS21);			// CTC + Prescaler 256 --> CPU/256
	//TCCR2 = (1<<WGM21)|(1<<CS21);			// CTC + Prescaler 8 --> CPU/8
	// bei 16MHz und prescaler 256 --> 2ms --> OCR2  = 124;
	// bei 1MHz und prescaler 8 --> 2ms --> OCR2  = 249;
    OCR2  = 124;							//vorladen, f_comp=fosc/(prescaler*(OCRnX+1)); OCRnX=fosc/(f_comp*prescaler)-1
    TIMSK |= (1<<OCIE2);        			//Output Compare Match aktivieren


	
	timer_flag = 0;

	uart_clear();


	sei();
	sms_init();
	sms_time (&year, &month, &day, &hour, &minute, &second);
	sprintf(s,"%02d%02d%02d-%02d:%02d:%02d",day, month, year, hour, minute, second);
	lcd_sends(s);
	
	sms_display_text("Das ist \n ein Test");

	while (1) {

		state=sms_state_machine();
		if (state==SMS_RETRIEVE_TELNR) {
			PORTD ^= ( 1 << PD6 );
			lcd_set_cursor(0,2);
			lcd_sends(sms_last_caller);
		}		

		// Timer1 ist abgelaufen		
		if ( timer_flag==1 ) {	
			PORTD ^= ( 1 << PD5 );
			error=ds75lv_get_temp(&vor,&nach);
			if (error!=true) {				
				ds75lv_temptoa(vor,nach,s);			// Zielstring muss mind. 10Bytes haben (z.B. '-125.1250'+'\n')
				lcd_set_cursor(7,2);
				lcd_sends(s);				
			}
			timer_flag=0;
		}
	}
return 0;
}



// bei 16MHz und prescaler 1024 --> 1s
ISR (TIMER1_COMPA_vect)
{
	if ( timer_flag==0) {timer_flag=1;}
	OCR1A  = 15624;				//f_comp=fosc/(prescaler*(OCRnX+1))
}

// bei 16MHz und prescaler 256 --> 2ms
ISR (TIMER2_COMP_vect)
{
	OCR2  = 124;				//f_comp=fosc/(prescaler*(OCRnX+1))
	uart_timeout++;
	if (uart_timeout>250) {
		uart_timeout=0;
	}
}



